<?php

namespace App\Service;

use App\Entity\Address;
use App\Entity\Email;
use App\Entity\Kitchen;
use App\Entity\Phone;
use App\Entity\Restaurant;
use App\Entity\RestaurantType;
use App\Entity\Tags;
use App\Mapper\RestaurantMapper;
use App\Model\File;
use App\Model\KitchenType;
use App\Model\Pagination;
use App\Model\RestaurantDetailElement;
use App\Model\RestaurantFilterVariants;
use App\Model\RestaurantList;
use App\Model\RestaurantListingElement;
use App\Model\RestaurantType as RestaurantTypeModel;
use App\Repository\Interface\KitchenRepositoryInterface;
use App\Repository\Interface\RestaurantRepositoryInterface;
use App\Repository\Interface\RestaurantTypeRepositoryInterface;
use Ramsey\Collection\Collection;

class RestaurantService
{
    public function __construct(
        private RestaurantRepositoryInterface $restaurantRepository,
        private RestaurantTypeRepositoryInterface $restaurantTypeRepository,
        private KitchenRepositoryInterface $kitchenRepository,
    ) {}

    public function getRestaurants(
        $page,
        $limit,
        $restaurantTypeId,
        $kitchenId): RestaurantList
    {
        $restaurants = $this->restaurantRepository->getAll(
            $page,
            $limit,
            $restaurantTypeId,
            $kitchenId);
        $count = $this->restaurantRepository->getCount();
        $restaurantTypes = $this->restaurantTypeRepository->getAll();
        $kitchens = $this->kitchenRepository->getAll();
        return new RestaurantList(
            new Pagination($page, ceil($count / $limit), $limit),
            new Collection(RestaurantListingElement::class, array_map(
            function (Restaurant $restaurant) {
                return RestaurantMapper::mapToListElement($restaurant);
            }, $restaurants)),
            new RestaurantFilterVariants(
                new Collection(
                    RestaurantTypeModel::class, array_map(
                        function (RestaurantType $restaurantType) {
                            return RestaurantMapper::mapToRestaurantType($restaurantType);
                        }, $restaurantTypes
                    )
                ),
                new Collection(
                    KitchenType::class, array_map(
                        function (Kitchen $kitchen) {
                            return RestaurantMapper::mapToKitchenType($kitchen);
                        }, $kitchens
                    ),
                )
            )
        );
    }

    public function getRestaurant(int $id): RestaurantDetailElement
    {
        $restaurant = $this->restaurantRepository->getById($id);
        return RestaurantMapper::mapToDetailElement($restaurant);
    }
}